

package Mail::SpamAssassin::Plugin::ResourceLimits;

use Mail::SpamAssassin::Plugin ();
use Mail::SpamAssassin::Logger ();
use Mail::SpamAssassin::Util   ();
use Mail::SpamAssassin::Constants qw(:sa);

use strict;
use warnings;

use BSD::Resource qw(RLIMIT_RSS RLIMIT_AS RLIMIT_CPU);

our @ISA = qw(Mail::SpamAssassin::Plugin);

sub new {
    my $class        = shift;
    my $mailsaobject = shift;

    $class = ref($class) || $class;
    my $self = $class->SUPER::new($mailsaobject);
    bless( $self, $class );

    $self->set_config( $mailsaobject->{conf} );
    return $self;
}

sub set_config {
    my ( $self, $conf ) = @_;
    my @cmds = ();

    push(
        @cmds,
        {
            setting  => 'resource_limit_mem',
            is_admin => 1,
            default  => '0',
            type     => $Mail::SpamAssassin::Conf::CONF_TYPE_NUMERIC
        }
    );

    push(
        @cmds,
        {
            setting  => 'resource_limit_cpu',
            is_admin => 1,
            default  => '0',
            type     => $Mail::SpamAssassin::Conf::CONF_TYPE_NUMERIC
        }
    );

    $conf->{parser}->register_commands( \@cmds );
}

sub spamd_child_init {
    my ($self) = @_;

    # Set CPU Resource limits if they were specified.
    Mail::SpamAssassin::Util::dbg("resourcelimitplugin: In spamd_child_init");
    Mail::SpamAssassin::Util::dbg( "resourcelimitplugin: cpu limit: " . $self->{main}->{conf}->{resource_limit_cpu} );
    if ( $self->{main}->{conf}->{resource_limit_cpu} ) {
        BSD::Resource::setrlimit( RLIMIT_CPU, $self->{main}->{conf}->{resource_limit_cpu}, $self->{main}->{conf}->{resource_limit_cpu} )
          || info("resourcelimitplugin: Unable to set RLIMIT_CPU");
    }

    # Set  Resource limits if they were specified.
    Mail::SpamAssassin::Util::dbg( "resourcelimitplugin: mem limit: " . $self->{main}->{conf}->{resource_limit_mem} );
    if ( $self->{main}->{conf}->{resource_limit_mem} ) {
        BSD::Resource::setrlimit( RLIMIT_RSS, $self->{main}->{conf}->{resource_limit_mem}, $self->{main}->{conf}->{resource_limit_mem} )
          || info("resourcelimitplugin: Unable to set RLIMIT_RSS");
        BSD::Resource::setrlimit( RLIMIT_AS, $self->{main}->{conf}->{resource_limit_mem}, $self->{main}->{conf}->{resource_limit_mem} )
          || info("resourcelimitplugin: Unable to set RLIMIT_AS");
    }
}

1;
